﻿using System;
using System.Collections;
using System.Collections.Generic;
using RPGM.Core;
using RPGM.Gameplay;
using RPGM.UI;
using UnityEngine;
using TMPro;

namespace RPGM.Gameplay
{
    /// <summary>
    /// This class provides all the data you need to control and change gameplay.
    /// </summary>
    [Serializable]
    public class GameModel
    {
        public CharacterController2D player;
        public DialogController dialog;
        public InputController input;
        public InventoryController inventoryController;
        public ShopController shopController;
        public MusicController musicController;

        public TextMeshPro gameStats;

        Dictionary<GameObject, HashSet<string>> conversations = new Dictionary<GameObject, HashSet<string>>();

        Dictionary<string, int> inventory = new Dictionary<string, int>();
        Dictionary<string, Sprite> inventorySprites = new Dictionary<string, Sprite>();

        HashSet<string> storyItems = new HashSet<string>();

        public IEnumerable<string> InventoryItems => inventory.Keys;

        Dictionary<string, int> shop = new Dictionary<string, int>();

        Dictionary<string, int> shopPrices = new Dictionary<string, int>();

        Dictionary<string, string> shopNames = new Dictionary<string, string>();
        Dictionary<string,ShopItem> shopItemsToBuy = new Dictionary<string, ShopItem>();

        public IEnumerable<string> ShopItems => shop.Keys;

        public GameObject GearAnimation = null;  // UnityEngine.GameObject.FindWithTag("GearAnimation").SetActive(true);

        // Extensions:
        public int money = 100;
        // World time.
        public float world_time_min = 0.0f;
        // Satisfaction level of park.
        public int satisfaction_level = 100;

        public int number_of_tenants = 3;

        public void ShowGameState() {
            string stats_text = "" + GameTimeToHumanText();
            stats_text += "\nMoney: " + money + "$";
            stats_text += "\nSatisfcation: " + satisfaction_level;
            stats_text += "\nTenants: " + number_of_tenants;
            gameStats.SetText(stats_text);
            // var gs = GameObject.FindObject("GameStats");
        }

        public void ChangeMoney(int delta)
        {
            money += delta;
        }

        public void ChangeSatisfactionLevel(int delta)
        {
            satisfaction_level += delta;
        }

        public void SetNumberOfTenants(int number)
        {
            number_of_tenants = number;
        }

        public void AdvanceGameTime(int time_delta)
        {
            world_time_min += time_delta;
        }

        public void AdvanceGameTimeSec(float time_delta_sec)
        {
            world_time_min += time_delta_sec * 1.0f / 60;
        }

        public string GameTimeToHumanText()
        {
            DateTime dt = new DateTime(2020, 2, 27, 19, 20, 0);
            dt = dt.AddMinutes(world_time_min);
            return dt.ToString("F");
        }

        public string GetShopName(string name)
        {
            string s;
            shopNames.TryGetValue(name, out s);
            return s;
        }

        public int GetShopPrice(string name)
        {
            int s;
            shopPrices.TryGetValue(name, out s);
            return s;
        }

        public int GetShopCount(string name)
        {
            int c;
            shop.TryGetValue(name, out c);
            return c;
        }

        public void AddShopItem(ShopItem item)
        {
            int c = 0;
            shop.TryGetValue(item.name, out c);
            c += item.count;
            shopPrices[item.name] = item.price;
            shopNames[item.name] = item.name;
            shop[item.name] = c;
            shopController.Refresh();
        }

        public bool HasShopItem(string name, int count = 1)
        {
            int c = 0;
            shop.TryGetValue(name, out c);
            return c >= count;
        }

        public bool BuyShopItem(string itemName, int count)
        {
            int c = 0;
            shop.TryGetValue(itemName, out c);
            c -= count;
            if (c < 0) return false;
            if (money < GetShopPrice(itemName)) 
            {
                MessageBar.Show($"Not enough money. Earn some! Get some repair jobs.");
                return false;
            }
            ChangeMoney(-GetShopPrice(itemName));
            shop[itemName] = c;
            shopController.Refresh();
            AddInventoryItem(itemName);
            return true;
        }
        public Sprite GetInventorySprite(string name)
        {
            Sprite s;
            inventorySprites.TryGetValue(name, out s);
            return s;
        }

        public int GetInventoryCount(string name)
        {
            int c;
            inventory.TryGetValue(name, out c);
            return c;
        }

        public void AddInventoryItem(InventoryItem item)
        {
            int c = 0;
            inventory.TryGetValue(item.name, out c);
            c += item.count;
            inventory[item.name] = c;
            inventoryController.Refresh();
        }

        public void AddInventoryItem(string inventoryName)
        {
            int c = 0;
            inventory.TryGetValue(inventoryName, out c);
            //inventorySprites[inventoryName] = item.sprite;
            inventory[inventoryName] = ++c;
            inventoryController.Refresh();
        }

        public bool HasInventoryItem(string name, int count = 1)
        {
            int c = 0;
            inventory.TryGetValue(name, out c);
            return c >= count;
        }

        public bool RemoveInventoryItem(string item_name, int count)
        {
            int c = 0;
            inventory.TryGetValue(item_name, out c);
            c -= count;
            if (c < 0) return false;
            inventory[item_name] = c;
            inventoryController.Refresh();
            return true;
        }

        public void ShowStore()
        {
            if (shopController.IsLoaded()) {
                Debug.Log("Shop is loaded, no need again.");
                shopController.Refresh();
                shopController.Show();
                return;
            }
            inventorySprites = new Dictionary<string, Sprite>();
            Sprite[] storeSprites = Resources.LoadAll<Sprite>("ShopItems");
            foreach (Sprite spr in storeSprites)
            {
                string itemName = GameConfig.ShopItemNameConvert(spr.name.Substring(10));
                inventorySprites.Add(itemName, spr);
                ShopItem item = new ShopItem();
                item.name = itemName;
                item.sprite = spr;
                item.price = Mathf.RoundToInt(UnityEngine.Random.Range(0f,25f));
                item.count = Mathf.RoundToInt(UnityEngine.Random.Range(1f,5f));
                AddShopItem(item);
            }
            input.ChangeState(InputController.State.ShopControl);
            Debug.Log("Loading shop again.");
            shopController.Refresh();
            shopController.SetLoaded();
        }

        public void RegisterStoryItem(string ID)
        {
            storyItems.Add(ID);
        }

        public bool HasSeenStoryItem(string ID)
        {
            return storyItems.Contains(ID);
        }

        public void RegisterConversation(GameObject owner, string id)
        {
            if (!conversations.TryGetValue(owner, out HashSet<string> ids))
                conversations[owner] = ids = new HashSet<string>();
            ids.Add(id);
        }

        public bool HasHadConversationWith(GameObject owner, string id)
        {
            if (!conversations.TryGetValue(owner, out HashSet<string> ids))
                return false;
            return ids.Contains(id);
        }

        public bool HasMet(GameObject owner)
        {
            return conversations.ContainsKey(owner);
        }
    }
}