using System.Collections.Generic;
using RPGM.Core;
using RPGM.Gameplay;
using UnityEngine;
using RPGM.UI;
using TMPro;

namespace RPGM.Gameplay
{
    /// <summary>
    /// Main class for implementing NPC game objects.
    /// </summary>
    public class NPCController : MonoBehaviour
    {
        public ConversationScript[] conversations;

        Quest activeQuest = null;

        GameModel model = Schedule.GetModel<GameModel>();
        public int satisfaction_level = 100;

        public DialogController notification;

        //private Dictionary<string, Sprite> notify_sprites = new Dictionary<string, Sprite>();

        void OnEnable()
        {
            // quests = UnityEngine.Object.FindObjectsOfType<Quest>();
            //quests = gameObject.GetComponents<Quest>();
            //quests = gameObject.GetComponents(Quest);
            //  gameObject.
            // quests = gameObject.GetComponentsInChildren<Quest>();
            //notify_sprites["active"] = Resources.Load("_GoldenApple", typeof(Sprite)) as Sprite;
            //Debug.Log("Sprite: " + notify_sprites["active"]);
            //notify_sprites["in progress"] = Resources.Load("_Chicken", typeof(Sprite)) as Sprite;
        }

        public Quest[] GetQuests()
        {
            return UnityEngine.Object.FindObjectsOfType<Quest>();
        }

        public Quest GetActiveQuest()
        {
            return activeQuest;
        }

        public bool HasActiveQuest()
        {
            return activeQuest != null;
        }

        public void CheckExpirationOfQuest()
        {
            var expires_in_min = activeQuest.ExpiresInMin(this);
            if (expires_in_min == 0) {
                activeQuest.SetFailed(this);
            } else {
                if (activeQuest.isStarted(this)) {
                    SwitchQuestStatusOnNpc("in progress", expires_in_min);
                } else {
                    SwitchQuestStatusOnNpc("active", expires_in_min);
                }
            }
        }

        public void AssignQuest()
        {
            // Let's try to create random quest.
            int rnd = Random.Range(0, GetQuests().Length);
            activeQuest = GetQuests()[rnd];
            activeQuest.SetEnabled(this);
        }

        public void MaybeAssignQuest()
        {
            float rnd = Random.Range(0.0f, 100.0f);
            if (rnd < (GameConfig.new_quest_probability)) {
                AssignQuest();
                SwitchQuestStatusOnNpc("active", activeQuest.waiting_time_min);
            }
        }

        public void DisableQuest()
        {
            activeQuest = null;
            SwitchQuestStatusOnNpc("disable");
        }

        public void SetInProgress()
        {
            SwitchQuestStatusOnNpc("in progress");
        }

        public void HideNpc()
        {
            if (gameObject.GetComponent<Renderer>().enabled == false) return;
            // gameObject.SetActive(false);
            gameObject.GetComponent<Renderer>().enabled = false;
            // Disable collider.
            gameObject.GetComponent<BoxCollider2D>().enabled = false;
            // Disable shadow.
            //foreach (var r in gameObject.GetComponentsInChildren<Renderer>()) {
            //    r.enabled = false;
            //}
        }

        public void ShowNpc()
        {
            if (gameObject.GetComponent<Renderer>().enabled == true) return;
            // gameObject.SetActive(true);
            gameObject.GetComponent<Renderer>().enabled = true;
            // Enable collider.
            gameObject.GetComponent<BoxCollider2D>().enabled = true;
            // Enable shadow.
            //foreach (var r in gameObject.GetComponentsInChildren<Renderer>()) {
            //    r.enabled = true;
            //}
        }

        public void AddNpc()
        {
            gameObject.SetActive(true);
        }

        public void RemoveNpc()
        {
            gameObject.SetActive(false);
        }

        public bool IsActive()
        {
            return gameObject.activeSelf;
        }

        private string HumanWaitTime(int time_min)
        {
            if (time_min > 60) return time_min / 60 + "h";
            return time_min + "m";
        }
        public void SwitchQuestStatusOnNpc(string status, int time_min = 0)
        {
            var sr2 = this.GetComponentInChildren<TextMeshPro>();
            sr2.SetText(status);
            if (status == "active") {
                ShowNpc();
                sr2.SetText("? " + HumanWaitTime(time_min));
            }
            if (status == "in progress") {
                sr2.SetText("! " + HumanWaitTime(time_min));
            }
            if (status == "disable") {
                HideNpc();
                sr2.SetText("");
            }
           if (status == "empty") {
                sr2.SetText("");
            }
        }

        public void OnCollisionEnter2D(Collision2D collision)
        {
            var c = GetConversation();
            if (c.Item1 != null)
            {
                var ev = Schedule.Add<Events.ShowConversation>();
                ev.conversation = c.Item1;
                ev.npc = this;
                ev.gameObject = gameObject;
                ev.conversationItemKey = c.Item2;
            }
        }

        public void ShowProgress()
        {
            Debug.Log("Start to show progress");
            SwitchQuestStatusOnNpc("empty");
            model.input.ChangeState(InputController.State.Pause);
            model.GearAnimation.SetActive(true);
        }

        public void StopProgress()
        {
            SwitchQuestStatusOnNpc("empty");
            model.input.ChangeState(InputController.State.CharacterControl);
            model.GearAnimation.SetActive(false);
        }

        public void ChangeSatisfactionLevel(int satisfaction_delta) {
            satisfaction_level += satisfaction_level;
            // TODO: we can do some nasty things here.
        }

        public void CompleteQuest(Quest q = null)
        {
            if (q == null) q = activeQuest;
            if (activeQuest != q) throw new System.Exception("Completed quest is not the active quest.");
            foreach (var i in activeQuest.requiredItems)
            {
                model.RemoveInventoryItem(i.item_name, i.count);
            }
            activeQuest.RewardItemsToPlayer();
            activeQuest.OnFinishQuest(this);
            // satisfaction_level += activeQuest.satisfaction_delta_npc;
            activeQuest = null;
        }

        public void StartQuest(Quest q)
        {
            if (activeQuest != null && activeQuest != q) throw new System.Exception("Only one quest should be active.");
            activeQuest = q;
        }

        (ConversationScript, string) GetConversation()
        {
            if (activeQuest == null)
            {
                // MaybeAssignQuest();
                if (activeQuest == null) {
                    return (Utils.EmptyConversation(gameObject), "");
                }
            }
            foreach (var q in GetQuests())
            {
                if (q == activeQuest)
                {
                    if (q.IsQuestComplete(this))
                    {
                      CompleteQuest(q);
                      return (q.getQuestInProgressConversation(), "FINAL");
                    }
                    if (q.isStarted(this))
                    {
                        return (q.getQuestInProgressConversation(), "CHECK_TOOLS");
                    }
                    q.SetStarted(this);
                    return (q.getQuestInProgressConversation(), "INITIAL");
                }
            }
            return (null, null);
        }
    }
}
