﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using RPGM.Core;
using RPGM.Gameplay;

public class GlobalTimerUpdate : MonoBehaviour
{

    private int GAME_MINUTES_PER_SECOND = 1;

    private int nextUpdate = 1;

    private GameModel gameModel;

    private NPCController[] allNpcs;

    private bool is_paused = false;

    private float real_time_finish_sec = 0.0f;
    private float real_time_start_sec = 0.0f;
    private float real_time_fix_sec = 0.0f;
    private float game_time_fix_min = 0.0f;
    private float last_update_fix_sec = 0.0f;

    void Awake()
    {
        gameModel = Schedule.GetModel<GameModel>();
    }
    // Start is called before the first frame update
    void Start()
    {
        allNpcs = UnityEngine.Object.FindObjectsOfType<NPCController>();
        for (var i = 0; i < GameConfig.intial_npcs_count; ++i) {
            allNpcs[i].AddNpc();
        }
        for (var i = GameConfig.intial_npcs_count; i < allNpcs.Length; ++i) {
            allNpcs[i].RemoveNpc();
        }
    }

    void Update()
    {
        // Not sure which one is better.
        // Super not optimized part.
        //var npcs = UnityEngine.Object.FindObjectsOfType<NPCController>();
        //Debug.Log("Just update: " + npcs.Length);        
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        if (Time.time >= nextUpdate && is_paused == false) {
            nextUpdate = Mathf.FloorToInt(Time.time) + 1;
            UpdateState();
        }
        if (is_paused == true) {
            var current_secs = Time.time;
            // We are fixing something, let's do nasty things.
            var secs_since_last_update = current_secs - last_update_fix_sec;
            last_update_fix_sec = current_secs;
            var delta_secs =
                (game_time_fix_min * 60) * (secs_since_last_update / real_time_fix_sec);
            gameModel.AdvanceGameTimeSec(delta_secs);
            gameModel.ShowGameState();
        }
        // Super not optimized part.

    }

    public void PauseTimer()
    {
        is_paused = true;
    }

    public void UnpauseTimer()
    {
        is_paused = false;
    }

    public void SpeedTimer(float real_time_delta_sec, int game_time_delta_min)
    {
        real_time_start_sec = Mathf.FloorToInt(Time.time);
        real_time_finish_sec = real_time_start_sec + real_time_delta_sec;
        real_time_fix_sec = real_time_delta_sec;
        game_time_fix_min = game_time_delta_min;
        last_update_fix_sec = real_time_start_sec;
    }

    void UpdateState()
    {
        gameModel.AdvanceGameTime(GAME_MINUTES_PER_SECOND);

        MaybeAddNpc();
        MaybeRemoveNpc();

        var npcs = UnityEngine.Object.FindObjectsOfType<NPCController>();
        foreach (var npc in npcs) {
            if (npc.HasActiveQuest()) {
                // Check expiration here.
                npc.CheckExpirationOfQuest();
            } else {
                npc.MaybeAssignQuest();
            }
        }

        gameModel.SetNumberOfTenants(npcs.Length);
        //GameObject.FindObjectsOfType<GameModel>();
        //UnityEngine.Object.FindObjectOfType<GameModel>().ShowGameState();
        gameModel.ShowGameState();
    }

    void MaybeAddNpc()
    {
        if (GameConfig.IsAddTenant(gameModel.satisfaction_level)) {
            Debug.Log("Adding new tenant...");
            foreach (var npc in allNpcs) {
                if (npc.IsActive() == false) {
                    npc.AddNpc();
                    return;
                }
            }
        }
    }

    void MaybeRemoveNpc()
    {
        if (GameConfig.IsRemoveTenant(gameModel.satisfaction_level)) {
            var npcs = UnityEngine.Object.FindObjectsOfType<NPCController>();
            foreach (var npc in npcs) {
                if (!npc.HasActiveQuest()) {
                    npc.RemoveNpc();
                    return;
                }
            }
        }
    }
}
