﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using RPGM.Gameplay;

public static class Utils
{
    public static ConversationPiece PieceBuilder(string id, string text, Quest quest = null, List<ConversationOption> options = null) {
        ConversationPiece piece = new ConversationPiece();
        piece.id = id;
        piece.text = text;
        piece.quest = quest;
        if (options == null) {
            piece.options = new List<ConversationOption>{};
        } else {
            piece.options = options;
        }
        return piece;
    }

    public static ConversationOption OptionBuilder(string text, string target_id) {
        ConversationOption option = new ConversationOption();
        option.text = text;
        option.targetId = target_id;
        option.enabled = true;
        return option;
    }

    public static ConversationPiece CreateFinalConversationPiece(Quest quest)
    {
        return PieceBuilder("FINAL", "Thank you! Here is your tip: " + quest.tip + "$");
    }

    public static ConversationPiece CreateWaitConversationPiece(Quest quest)
    {
        return PieceBuilder("WAIT",
            "Thank you. Will wail for it to finish!",
            quest
        );
    }

    public static ConversationPiece CreateNoConversationPiece()
    {
        return PieceBuilder("REJECT",
            "Oh, this is super sad!"
        );
    }

    public static ConversationPiece CreateWaitConversationPiece()
    {
        return PieceBuilder("WAIT",
            "Ok, I will wait!");
    }

    public static ConversationPiece CreateDoItNowPiece(Quest quest)
    {
        var yes_option = OptionBuilder("Yes", "FINAL");
        var no_option = OptionBuilder("No", "WAIT");

        var text = "Wanna do it right now?";
        text += "\nTime: " + quest.required_time + " minutes.";

        return PieceBuilder("DO_IT_NOW",
            text,
            null,
            new List<ConversationOption> { yes_option, no_option }
        );
    }

    public static ConversationPiece CreateNotEnoughTools(Quest quest)
    {
        var yes_option = OptionBuilder("I will come back later", "WAIT");
        var no_option = OptionBuilder("Sorry, can't do it", "REJECT");

        var text = "You don't have enough tools.";
        // var missingTools = quest.MissingTools();
        // var missingToolsStr = "\nMissing tools: ";
        // foreach (var tool in missingTools)
        // {
        //     missingToolsStr += tool + ", ";
        // }
        // text += missingToolsStr;

        return PieceBuilder("NOT_ENOUGH_TOOLS",
            text,
            null,
            new List<ConversationOption> {yes_option, no_option});
    }

    public static ConversationPiece CreateCheckToolsPiece()
    {
        var good_option = OptionBuilder("<NO DISPLAY>", "DO_IT_NOW");
        var bad_option = OptionBuilder("<NO DISPLAY>", "NOT_ENOUGH_TOOLS");

        return PieceBuilder("CHECK_TOOLS",
            "<NO DISPLAY>"
        );
    }

    public static ConversationPiece CreateInitialConversationPiece(Quest quest)
    {
        ConversationPiece piece = new ConversationPiece();
        piece.id = "INITIAL";
        piece.text = quest.desc;
        string required_items_str = "";
        foreach (var item in quest.requiredItems) {
            required_items_str += item.item_name + ", ";
        }
        piece.text += "\nTools: " + required_items_str;
        piece.text += "\nIt will take " + quest.required_time + " minutes";
        piece.text += "\nSatisfaction level " + quest.satisfaction_delta;
        piece.quest = null;
        
        var yes_option = OptionBuilder("Yes!", "CHECK_TOOLS");
        var no_option = OptionBuilder("No!", "REJECT");

        piece.options = new List<ConversationOption> {};
        piece.options.Add(yes_option);
        piece.options.Add(no_option); // { yes_option, no_option };

        return piece;
    }

    public static ConversationScript CreateQuestConversation(Quest quest, ConversationScript script)
    {
        if (script.initialized_for_quest) return script;
        //script.Add(CreateYesConversationPiece(quest));
        script.Add(CreateInitialConversationPiece(quest));  // INITIAL
        script.Add(CreateWaitConversationPiece(quest));  // WAIT
        script.Add(CreateNoConversationPiece());  // REJECT
        script.Add(CreateWaitConversationPiece());  // WAIT
        script.Add(CreateDoItNowPiece(quest));  // DO_IT_NOW
        script.Add(CreateFinalConversationPiece(quest));  // FINAL
        script.Add(CreateNotEnoughTools(quest));  // NOT_ENOUGH_TOOLS
        script.Add(CreateCheckToolsPiece());  // CHECK_TOOLS
        //script.Add(CreateNoConversationPiece());
        //script.Set(script.Get("NO"), CreateNoConversationPiece());
        //script.Set(script.Get("INITIAL"), CreateInitialConversationPiece(quest));
        script.SetInitializedForQuest();
        return script;
    }

    public static ConversationScript CreateFinalQuestConversation(Quest quest, ConversationScript script)
    {
        if (script.initialized_for_quest) return script;
        script.Add(CreateFinalConversationPiece(quest));
        //script.Set(script.Get("FINAL"), CreateFinalConversationPiece(quest));
        script.SetInitializedForQuest();
        return script;
    }

    public static ConversationScript EmptyConversation(GameObject gameObject)
    {
        var sc = gameObject.AddComponent<ConversationScript>();
        sc.Add(PieceBuilder("EMPTY", "I don't have tasks for you now."));
        return sc;
    }
}
